
# q-expansion of newform 2100.4.a.j, downloaded from the LMFDB on 14 March 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2100
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [3], [0], [-7], [-26], [39], [101], [-48], [-57], [-291], [-79], [-322], [455], [307], [-508], [31], [211], [-797], [924], [212], [-22], [874], [587], [-266], [-1822], [-1872], [-1595], [-1026], [-588], [-22], [1750], [1664], [-1080], [-896], [-591], [-1606], [-3554], [2897], [66], [-1672], [4710], [2842], [-4197], [-2662], [-3601], [-1840], [-1733], [-4727], [-2863], [5578], [-5016], [2992], [-1574], [-5697], [-2599], [1115], [4194], [-3972], [1842], [-2680], [-2680], [5656], [-6998], [5610], [-166], [-5333], [6481], [-9221], [-1910], [3409], [-2130], [647], [11503], [3692], [8103], [2664], [-2050], [1213], [10876], [-12534], [4569], [-9262], [-9429], [-418], [-419], [9628], [14052], [-12089], [-2744], [-592], [-3361], [8604], [-9826], [4758], [-20631], [7566], [12028], [21317], [-15416], [17520], [-17379], [-15830], [7343], [-10806], [-1593], [-9314], [3083], [18882], [1707], [1440], [8284], [24042], [-862], [9890], [-21646], [19658], [23818], [-28490], [-11002], [11506], [8398], [-1901], [-19896], [15662], [1162], [13501], [-20672], [-7098], [4053], [-10331], [-8969], [11613], [-32476], [29914], [-3090], [-7760], [-34020], [15154], [13386], [36468], [-38422], [19194], [7426], [33170], [16397], [14996], [21053], [6390], [22176], [42472], [-3472], [32639], [45513], [27626], [11561], [25773], [-13240], [-30653], [-19030], [-5320], [-14524], [8688], [19672], [45996], [32454], [-34048], [-14804], [35546], [9295], [-43292], [-4234], [-12907], [54339], [-42009], [-24388], [-533], [-42644], [50631], [-24499], [19387], [19478], [14151], [-60046], [62639], [54552], [-51426], [16516], [-7522], [-60479], [-8340], [-47040], [22864], [14920], [34754], [46884], [45303], [23845], [26242], [-22914], [41372], [44542], [-16441], [-16119], [50274], [-39921], [26336], [-9091], [-76170], [-29706], [16838], [1518], [33882], [64318], [-3061], [21270], [14384], [-47900], [58242], [-95663], [43167], [-98512], [-30787], [-80226], [69540], [-85420], [-1186], [-73382], [65744], [55996], [-86902], [-59060], [10776], [-97184], [-35358], [-60944], [14160], [17098], [10406], [-30646], [-44306], [-20660], [-17182], [-35529], [-103082], [-58887], [110904], [-58267], [68460], [16748], [3368], [-27824], [65451], [20745], [103410], [-70800], [-2486], [52214], [-52646], [51678], [80922], [-12972], [65620], [73069], [130022], [-67551], [53426], [-107211], [-44713], [-23177], [118131], [83913], [-89619], [-45028], [54096], [-104620], [-54772], [-111256], [24686], [-30345], [-66185], [-30728], [-122128], [143904], [-19629], [56468], [-42957], [-82181], [-151818], [129695], [-114258], [99390], [56996], [59476], [110600], [21318], [28860], [-116705], [-169562], [-138885], [27554], [-66852], [-96577], [-21974], [-47556], [97796], [-66507], [73166], [63009], [-108315], [80378], [-49866], [68193], [-97718], [21176], [-113205], [167114], [26244], [142263], [18012], [-67900], [-123122], [99286], [-67402], [-118644], [52206], [-68872], [-32149], [13170], [7048], [45843], [125606], [123699], [103087], [-95811], [-58537], [-16644], [-31436], [-30387], [-180134], [53345], [-51086], [-103370], [-190584], [93944], [59625], [14104], [158874], [-163631], [-60936], [96558], [163268], [-96741], [-31233], [145175], [-66114], [-49797], [-9294], [-54635], [196038], [32139], [3716], [66554], [-20416], [-153056], [-16886], [183121], [141202], [70036], [20953], [-226572], [174088], [-51833], [198135], [38993], [-29960], [-180049], [-37200], [141836], [214725], [-75840], [99382], [-155780], [-254388], [-77532], [-230040], [-26448], [236406], [141446], [242588], [190486], [20999], [177784], [203998], [130854], [41059], [137722], [-111869], [-178451], [-60636], [34434], [102507], [8322], [-58724], [32991], [-111776], [251719], [25082], [-41910], [-192918], [-153240], [72416], [-289998], [-149928], [173755], [-202213], [-208748], [261435], [-77229], [-16719], [-194362], [-103301], [161651], [123429], [-197215], [84696], [144628], [-15118], [-274746], [-230312], [-16006]]
